@@ -1,13 +1,27 @@
 from datetime import datetime as dt
 from datetime import timedelta as td
-from typing import Optional
+from os import getenv
 
 # libs
 from peewee import AutoField, TextField, ForeignKeyField, TimestampField
 from peewee import Model, PostgresqlDatabase
-from playhouse.postgres_ext import BinaryJSONField
+from playhouse.postgres_ext import IntegerField
+import psycopg2 as pg2
+import psycopg2.extras as pg2_extras
 
-db: Optional[PostgresqlDatabase] = None
+# app
+from src.easytrack.utils import notnull
+import dotenv
+
+dotenv.load_dotenv()
+
+db = PostgresqlDatabase(
+	host=notnull(getenv(key='POSTGRES_HOST')),
+	port=notnull(getenv(key='POSTGRES_PORT')),
+	database=notnull(getenv(key='POSTGRES_DBNAME')),
+	user=notnull(getenv(key='POSTGRES_USER')),
+	password=notnull(getenv(key='POSTGRES_PASSWORD'))
+)
 
 
 class User(Model):
@@ -93,7 +107,7 @@ class HourlyStats(Model):
 	participant = ForeignKeyField(Participant, on_delete='CASCADE', null=False)
 	data_source = ForeignKeyField(DataSource, on_delete='CASCADE', null=False)
 	ts = TimestampField(null=False)
-	amounts = BinaryJSONField()
+	amount = IntegerField(default=0)
 
 	class Meta:
 		database = db
@@ -104,3 +118,30 @@ class HourlyStats(Model):
 			(('participant', 'data_source'), False),  # selection by participant and/or data source
 			(('ts',), False),  # selection by timestamp
 		)
+
+
+# create schema if necessary
+conn = pg2.connect(
+	host=notnull(getenv(key='POSTGRES_HOST')),
+	port=notnull(getenv(key='POSTGRES_PORT')),
+	dbname=notnull(getenv(key='POSTGRES_DBNAME')),
+	user=notnull(getenv(key='POSTGRES_USER')),
+	password=notnull(getenv(key='POSTGRES_PASSWORD'))
+)
+cur: pg2_extras.DictCursor = conn.cursor(cursor_factory=pg2_extras.DictCursor)
+cur.execute('create schema if not exists core')
+cur.execute('create schema if not exists data')
+cur.execute('create schema if not exists geoplan')
+conn.commit()
+conn.close()
+# connect and prepare tables
+db.connect()
+db.create_tables([
+	User,
+	Campaign,
+	DataSource,
+	CampaignDataSources,
+	Supervisor,
+	Participant,
+	HourlyStats
+])
